%%%%%%%%%%%%%%%%%
% Investigate the effects of permanent shock on steady state equilibrium
% For two kinds of robustness checks
%  Given obtained path of shocks, we assume 
% 1. s (sale frequency) is constant because firms do not adjust it optimally 
% 2. mu (magnitude of sales) changes by changes in structural parameters
% for Figure 5

%%%%%%%%%%%%%%%%%

clc
clear

%
set(0,'defaultAxesFontSize',12);
set(0,'defaultAxesFontName','century');
set(0,'defaultTextFontSize',12);
set(0,'defaultTextFontName','century');
%

%% robustness check 1

% initialization
% Targeted variables
s = 0.211;
mu = 0.845;
chi = 2.055;

%%
% Parameters of model of sales
tmp0 = matchtargets(mu,chi,s,s);
epsilon = tmp0(1)
eta = tmp0(2)
lambda = tmp0(3)
sigma = tmp0(4)

% Other parameters (monthly calibration)
beta = (0.9975); % subjective discount factor
thetac = 0.333; % intertemporal elasticity of substitution in consumption
thetah = 0.7; % Frisch elasticity of labour supply
zeta = 20; % elasticity of substitution bw differentiated labour inputs
alpha = 0.667; % elasticity of output w.r.t. hours
gamma = (1-alpha)/alpha; % elasticity of marginal cost w.r.t. output
thetal = 1.7; 

param = [mu chi s epsilon eta lambda sigma beta thetac thetah zeta alpha gamma thetal];

tmp1 = equilaux(epsilon,eta,lambda,mu,s);
psi = tmp1(1);
delta = tmp1(4);
phil = param_steady(epsilon, eta, lambda, mu, chi, s, psi, delta, thetal, alpha)

%% load main simulation results 
load yvec_results;
vshock = yvec(:,11);
clear yvec;

%%
% if flag == {1, 2, 3} then {e_a, e_h, e_g} is non-zero
flag = 3; 
v0 = zeros(1,11);
eval('run ..\macrodata.m') % load actual vh
% from 1981:2Q to 2013:3Q
% repeat every year (every 4 quarters)
nn = 2012-1982+1;

vh_data_m = vh - mean(vh);

vh_data = sum(vh_data_m(1:3))/3;
vy0 = sum(vy(1:3))/3;

[y fval] = fsolve(@(x) steadystate_change_func_robust1(param, vshock(1), x, flag), v0);
yvec1 = [y vshock(1) vy0];
for ii=1:nn
    vh_data = [vh_data sum(vh_data_m(4*(ii-1)+1:4*ii))/4];
    vy0 = sum(vy(4*(ii-1)+1:4*ii))/4;
    [y fval] = fsolve(@(x) steadystate_change_func_robust1(param, vshock(ii+1), x, flag), v0);
    yvec1 = [yvec1 ; y vshock(ii+1) vy0];
end
vh_data = [vh_data sum(vh_data_m(end-2:end))/3];
vy0 = sum(vy(end-2:end))/3;
[y fval] = fsolve(@(x) steadystate_change_func_robust1(param, vshock(end), x, flag), v0);
yvec1 = [yvec1 ; y vshock(end) vy0];

vs1 = yvec1(:,1);
vl1 = yvec1(:,2);
vmu1 = yvec1(:,4);
vy1 = yvec1(:,7);
vchi1 =  yvec1(:,9);
vsale1 =  yvec1(:,10);
vh1 = yvec1(:,11);
vy_data1 = yvec1(:,13); vy_data1 = vy_data1 - mean(vy_data1);

%% Add mean to sales-related variables

times= 1981:2013; 
times_sales= 1988:2013; % time when the value of sales actual data exist
tsample = size(times,2);
subsample = size(times,2)-size(times_sales,2)+1:tsample; % sample that has the value of sales actual data
subsample0 = 1:size(times,2)-size(times_sales,2); % sample that has no value of sales actual data

s_model1 = s*exp(vs1-mean(vs1(subsample)));
disc_model1 = (1-mu)*exp(-(vmu1-mean(vmu1(subsample))));
chi_model1 = chi*exp(vchi1-mean(vchi1(subsample)));
sale_model1 = s*chi/(1-s+s*chi)*exp(vsale1-mean(vsale1(subsample)));
l_model1 = lambda*exp(vl1-mean(vl1(subsample)));

elast_model1 = l_model1*epsilon + (1-l_model1)*eta;

%% load actual sales variables

load ..\sales_q.txt;
% sales_q is from 1988:2Q to 2013:3Q
nn = 2012-1989+1;
sales_a = sum(sales_q(1:3,:),1)/3;
for ii=1:nn
    sales_a = [sales_a; sum(sales_q(4*ii:4*ii+3,:),1)/4];
end
sales_a = [sales_a; sum(sales_q(end-2:end,:),1)/3];
s_data = [NaN.*subsample0'; s + sales_a(:,1) - mean(sales_a(:,1))];
disc_data = [NaN.*subsample0'; (1-mu) + sales_a(:,2)-mean(sales_a(:,2))];
chi_data = [NaN.*subsample0'; chi + sales_a(:,3) - mean(sales_a(:,3))];
sale_data = [NaN.*subsample0'; s*chi/(1-s+s*chi) + sales_a(:,4)-mean(sales_a(:,4))];

load 'steadystate_change'; % load benchmark simulation results

%% robustness check 2

% initialization
% Targeted variables
s = 0.211;
mu = 1-disc_data(size(subsample0,2)+1); %0.845;
lambda = lambda;
%chi = 2.055;

%%
% Parameters of model of sales
x0 = [epsilon eta chi sigma];
tmp0 = fminsearch(@(x) matchtargets_robust2(x,mu,lambda,s,s), x0);
epsilon = tmp0(1);
eta = tmp0(2);
chi = tmp0(3);
sigma = tmp0(4);

% Other parameters (monthly calibration)
beta = (0.9975); % subjective discount factor
thetac = 0.333; % intertemporal elasticity of substitution in consumption
thetah = 0.7; % Frisch elasticity of labour supply
zeta = 20; % elasticity of substitution bw differentiated labour inputs
alpha = 0.667; % elasticity of output w.r.t. hours
gamma = (1-alpha)/alpha; % elasticity of marginal cost w.r.t. output
thetal = 1.7; 

param = [mu chi s epsilon eta lambda sigma beta thetac thetah zeta alpha gamma thetal];

tmp1 = equilaux(epsilon,eta,lambda,mu,s);
psi = tmp1(1);
delta = tmp1(4);
%phil = param_steady(epsilon, eta, lambda, mu, chi, s, psi, delta, thetal, alpha)
s_mu_chi_eta_epsilon_l = [s mu chi eta epsilon lambda];

%%
eval('run ..\macrodata.m') % load actual vh
vh = vh - mean(vh);
vh0 = sum(vh(1:3))/3;
vy0 = sum(vy(1:3))/3;
[y fval] = fsolve(@(x) steadystate_change_func170206(param, vh0, x, flag), v0);
yvec2 = [y vh0 vy0];
nn = 2012-1982+1;
for ii=1:nn
    vh0 = sum(vh(4*(ii-1)+1:4*ii))/4;
    vy0 = sum(vy(4*(ii-1)+1:4*ii))/4;

    % initialization
% Targeted variables
if (ii+1) <= size(subsample0,2)
    mu = 1-disc_data(size(subsample0,2)+1);
else
    mu = 1-disc_data(ii+1); %0.845;
end

% Parameters of model of sales
x0 = [epsilon eta chi sigma];
tmp0 = fminsearch(@(x) matchtargets_robust2(x,mu,lambda,s,s), x0);
epsilon = tmp0(1);
eta = tmp0(2);
chi = tmp0(3);
sigma = tmp0(4);
param = [mu chi s epsilon eta lambda sigma beta thetac thetah zeta alpha gamma thetal];

tmp1 = equilaux(epsilon,eta,lambda,mu,s);
psi = tmp1(1);
delta = tmp1(4);
%phil = param_steady(epsilon, eta, lambda, mu, chi, s, psi, delta, thetal, alpha);
s_mu_chi_eta_epsilon_l = [s_mu_chi_eta_epsilon_l; s mu chi eta epsilon lambda];

    [y fval] = fsolve(@(x) steadystate_change_func170206(param, vh0, x, flag), v0);
    yvec2 = [yvec2 ; y vh0 vy0];
end
vh0 = sum(vh(end-2:end))/3;
vy0 = sum(vy(end-2:end))/3;

% initialization
% Targeted variables
mu = 1-disc_data(ii+1); %0.845;

% Parameters of model of sales
x0 = [epsilon eta chi sigma];
tmp0 = fminsearch(@(x) matchtargets_robust2(x,mu,lambda,s,s), x0);
epsilon = tmp0(1);
eta = tmp0(2);
chi = tmp0(3);
sigma = tmp0(4);
param = [mu chi s epsilon eta lambda sigma beta thetac thetah zeta alpha gamma thetal];

tmp1 = equilaux(epsilon,eta,lambda,mu,s);
psi = tmp1(1);
delta = tmp1(4);
%phil = param_steady(epsilon, eta, lambda, mu, chi, s, psi, delta, thetal, alpha);
s_mu_chi_eta_epsilon_l = [s_mu_chi_eta_epsilon_l; s mu chi eta epsilon lambda];

[y fval] = fsolve(@(x) steadystate_change_func170206(param, vh0, x, flag), v0);
yvec2 = [yvec2 ; y vh0 vy0];

vs2 = yvec2(:,1);
vl2 = yvec2(:,2);
vmu2 = yvec2(:,4);
vy2 = yvec2(:,7);
vchi2 =  yvec2(:,9);
vsale2 =  yvec2(:,10);
vshock2 = yvec2(:,11);
vh2 = yvec2(:,12);
vy_data2 = yvec2(:,13); vy_data2 = vy_data2 - mean(vy_data2);

%% Add mean to sales-related variables

s_model2 = s_mu_chi_eta_epsilon_l(:,1).*exp(vs2-mean(vs2(subsample)));
disc_model2 = (1-s_mu_chi_eta_epsilon_l(:,2)).*exp(-(vmu2-mean(vmu2(subsample))));
chi_model2 = s_mu_chi_eta_epsilon_l(:,3).*exp(vchi2-mean(vchi2(subsample)));
sale_model2 = s_mu_chi_eta_epsilon_l(:,1).*s_mu_chi_eta_epsilon_l(:,3)./(1-s_mu_chi_eta_epsilon_l(:,1)+s_mu_chi_eta_epsilon_l(:,1).*s_mu_chi_eta_epsilon_l(:,3)).*exp(vsale2-mean(vsale2(subsample)));
l_model2 = s_mu_chi_eta_epsilon_l(:,6).*exp(vl2-mean(vl2(subsample)));

elast_model2 = l_model2.*s_mu_chi_eta_epsilon_l(:,5) + (1-l_model1).*s_mu_chi_eta_epsilon_l(:,4);

%% Draw graph of sales
figure(1)
subplot(3,3,1)
h=plot(times, vh_data, times, vh_data, times, vh1, 'r-', times(1:5:31), vh1(1:5:31), 'r*', ...
    times, vh2, '-', times(1:5:31), vh2(1:5:31), '+'); 
title('Hours');
xlim([1980 2014]);
set(h(2),'linewidth',1.5);

subplot(3,3,2)
h=plot(times, s_data, times, result_benchmark(:,1), 'k-', times, s_model1, 'r-', times(1:5:31), s_model1(1:5:31), 'r*', ...
    times, s_model2, 'm-', times(1:5:31), s_model2(1:5:31), 'm+'); 
title('Freq of sales');
xlim([1980 2014]);
%ylim([0.1 0.3])
set(h(2),'linewidth',1.5);

subplot(3,3,3)
h=plot(times, disc_data, times, result_benchmark(:,2), 'k-', times, disc_model1, 'r-', times(1:5:31), disc_model1(1:5:31), 'r*', ...
    times, disc_model2, 'm-', times(1:5:31), disc_model2(1:5:31), 'm+');
title('Mag of sales');
xlim([1980 2014]);
%ylim([0.1 0.2])
set(h(2),'linewidth',1.5);

subplot(3,3,4)
h=plot(times, chi_data, times, result_benchmark(:,3), 'k-', times, chi_model1, 'r-', times(1:5:31), chi_model1(1:5:31), 'r*', ...
    times, chi_model2, 'm-', times(1:5:31), chi_model2(1:5:31), 'm+'); 
title('Q ratio');
xlim([1980 2014]);
set(h(2),'linewidth',1.5);

subplot(3,3,5)
h=plot(times, sale_data, times, result_benchmark(:,4), 'k-', times(1:5:31), sale_model1(1:5:31), 'r*', times(1:5:31), sale_model2(1:5:31), 'm+', ...
    times, sale_model1, 'r-', ... 
     times, sale_model2, 'm-'); 
title('Rev ratio');
xlim([1980 2014]);
%ylim([0.25 0.45])
set(h(2),'linewidth',1.5);
legend( {'Actual',  'Model (benchmark)','Model with $\hat{s}$=0', 'Model with variable $\mu$'}, 'Location','SouthEast', 'Interpreter','Latex')
%xlabel('$$\hat{x}$$','Interpreter','Latex')

subplot(3,3,6)
h=plot(times, result_benchmark(:,5), 'k-', times, l_model1, 'r-', times(1:5:31), l_model1(1:5:31), 'r*', ...
    times, l_model2, 'm-', times(1:5:31), l_model2(1:5:31), 'm+'); 
title('Loyal');
xlim([1980 2014]);
%ylim([0.86 0.92])
set(h(1),'linewidth',1.5);

% evaluate weighted average of elasticity
subplot(3,3,7)
h=plot(times, result_benchmark(:,6), 'k-', times, elast_model1, 'r-', times(1:5:31), elast_model1(1:5:31), 'r*', ...
    times, elast_model2, 'm-', times(1:5:31), elast_model2(1:5:31), 'm+'); 
title('Elasticity');
xlim([1980 2014]);
%ylim([3.8 4.2])
set(h(1),'linewidth',1.5);

% GDP
vy_model1 = vy1-mean(vy1(subsample));
dvy_model1 = vy_model1(2:end)-vy_model1(1:end-1);
dvy_data1 = vy_data1(2:end)-vy_data1(1:end-1);
vy_model2 = vy2-mean(vy2(subsample));
dvy_model2 = vy_model2(2:end)-vy_model2(1:end-1);
subplot(3,3,8)
h=plot(times(2:end), dvy_data1, times(2:end), result_benchmark(2:end,7), 'k-', times(2:end), dvy_model1, 'r-', times(2:5:31), dvy_model1(2:5:31), 'r*', ...
    times(2:end), dvy_model2, 'm-', times(2:5:31), dvy_model2(2:5:31), 'm+'); 
title('GDP chg');
xlim([1980 2014]);
set(h(2),'linewidth',1.5);

